/*******************************************************************************
 * Portions created by Sebastian Thomschke are copyright (c) 2005-2007 Sebastian
 * Thomschke.
 * 
 * All Rights Reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Sebastian Thomschke - initial implementation.
 *******************************************************************************/
package net.sf.oval.constraint;

import java.text.DateFormat;
import java.text.ParseException;
import java.util.Calendar;
import java.util.Date;

import net.sf.oval.Validator;
import net.sf.oval.configuration.annotation.AbstractAnnotationCheck;
import net.sf.oval.context.OValContext;

/**
 * @author Sebastian Thomschke
 */
public class PastCheck extends AbstractAnnotationCheck<Past>
{
	private static final long serialVersionUID = 1L;

	public boolean isSatisfied(final Object validatedObject, final Object value,
			final OValContext context, final Validator validator)
	{
		if (value == null) return true;

		// check if the value is a Date
		if (value instanceof Date)
		{
			// return ((Date) value).before(new Date());
			return ((Date) value).getTime() < System.currentTimeMillis();
		}

		// check if the value is a Calendar
		if (value instanceof Calendar)
		{
			// return ((Calendar) value).getTime().before(new Date());
			return ((Calendar) value).getTime().getTime() < System.currentTimeMillis();
		}

		// see if we can extract a date based on the object's String representation
		final String stringValue = value.toString();
		try
		{
			// return DateFormat.getDateTimeInstance().parse(stringValue).before(new Date());
			return DateFormat.getDateTimeInstance().parse(stringValue).getTime() < System
					.currentTimeMillis();
		}
		catch (final ParseException ex)
		{
			return false;
		}
	}
}
